<?php

namespace GunTab;

use Dragon\Config;
use GunTab\Api\GunTab\Client;
use GunTab\Api\GunTab\RequestObjects\Invoice;
use GunTab\Api\GunTab\RequestObjects\Listing;
use Dragon\Log;

class GunTabPaymentGateway extends \WC_Payment_Gateway {
	public $form_fields = [
		'enabled' => [
			'title'       => 'Enable/Disable',
			'label'       => 'Enable GunTab Gateway',
			'type'        => 'checkbox',
			'description' => '',
			'default'     => 'no'
		],
		'title' => [
			'title'       => 'Title',
			'type'        => 'text',
			'description' => 'This controls the title which the user sees during checkout.',
			'default'     => 'Pay with GunTab',
			'desc_tip'    => true,
		],
		'description' => [
			'title'       => 'Description',
			'type'        => 'textarea',
			'description' => 'This controls the description which the user sees during checkout.',
			'default'     => 'GunTab is a firearm-friendly escrow service that allows ' .
								'you to pay online with total safety and security.',
		],
		'testmode' => [
			'title'       => 'Test mode',
			'label'       => 'Enable Test Mode',
			'type'        => 'checkbox',
			'description' => 'Place the payment gateway in test mode by using the test API token.',
			'default'     => 'yes',
			'desc_tip'    => true,
		],
		'show_icon' => [
			'title'       => 'Show Icon?',
			'label'       => 'Show the icon during checkout',
			'type'        => 'checkbox',
			'description' => 'Uncheck this box if you don\'t want to show the ' .
								'GunTab logo on the checkout payment types ' .
								'list that customers will see.',
			'default'     => 'yes',
			'desc_tip'    => true,
		],
		'show_quick_checkout_buttons' => [
			'title'       => 'Show quick checkout buttons?',
			'label'       => 'Show the "Pay with GunTab" button on product detail pages',
			'type'        => 'checkbox',
			'description' => 'If you ever have customers who only want to buy one item, ' .
								'you should enable this feature to help them quickly checkout.',
			'default'     => 'no',
			'desc_tip'    => true,
		],
		'test_api_key' => [
			'title'       => 'Test API token',
			'type'        => 'text',
		],
		'prod_api_key' => [
			'title'       => 'Production API token',
			'type'        => 'text',
			'description' => 'IMPORTANT: Also ' .
				'remember to set your webhook callback for your ' .
				'<a href="https://www.guntab.com/webhooks/endpoints" target="_blank">endpoint</a>. ' .
				'The webhook for this site is ',
		],
	];

	public static function register(array $gateways) {
		$gateways[] = static::class;
		return $gateways;
	}

	public function __construct() {
		$url = Rest::getWebhookUrl();
		$this->form_fields['prod_api_key']['description'] .= '<code>' . $url . '</code>';

		$this->configureAdminSettings();
		$this->init_settings();
		$this->configureFrontEnd();

		add_action('woocommerce_update_options_payment_gateways_guntab', [$this, 'process_admin_options']);
	}

	public function process_payment($orderId) {
		global $woocommerce;
		$order = wc_get_order($orderId);

		$apiKey = $this->testmode ? $this->test_api_key : $this->prod_api_key;
		$client = new Client($apiKey);

		$invoice = $this->compileInvoice($order);
		$response = $client->createInvoice($invoice);
		if (!empty($response)) {

			$woocommerce->cart->empty_cart();
			$newStatus = get_option('guntab_woocommerce_status_before_redirect_to_escrow', 'wc-pending');
			$order->update_status($newStatus, __('Order sent to GunTab for payment', 'guntab'));

			Log::info('Redirected customer to payment screen for transaction ID ' . $response->id);
			return [
				'result' => 'success',
				'redirect' => $response->response_url,
			];

		} else {
			Log::info('[GUNTAB API ERROR] ' . print_r($client->getError(), true) .
					"\nINVOICE DATA SENT: " . print_r($invoice, true));

			wc_add_notice(__('Something went wrong when trying to create your ' .
					'invoice on GunTab, and has been logged in our system. ' .
					'Please contact customer service and let them know when ' .
					'this error occurred.', 'guntab'));
		}
	}

	private function configureAdminSettings() {
		$this->id = 'guntab';
		$this->icon = Config::$pluginBaseUrl . '/resources/img/guntab-icon.png';
		$this->has_fields = false;
		$this->method_title = 'GunTab Payments';
		$this->method_description = 'Accept payments with GunTab to protect your firearm and ammo transactions. This adds a "Pay with GunTab" button to your checkout page. Free to use.';

		$this->supports = [
			'products'
		];
	}

	private function configureFrontEnd() {
		$showIcon = $this->get_option('show_icon') === 'yes';

		$this->icon = $showIcon ? $this->icon : null;
		$this->title = $this->get_option('title');
		$this->description = $this->get_option('description');
		$this->enabled = $this->get_option('enabled');
		$this->testmode = $this->get_option('testmode') === 'yes';
		$this->test_api_key = $this->get_option('test_api_key');
		$this->prod_api_key = $this->get_option('prod_api_key');
	}

	private function compileInvoice(\WC_Order $order) {
		$invoice = new Invoice();
		$invoice->shipping_amount_cents = $order->get_shipping_total() * 100;
		$invoice->seller_order_id = (int)$order->get_id();
		$invoice->sales_tax_amount_cents = $order->get_total_tax() * 100;

		$total = 0;
		foreach($order->get_items() as $item) {
			$product = $item->get_product();

			$listing = new Listing();
			$listing->quantity = $item->get_quantity();

			$totalBucks = round(($order->get_line_total($item) / $listing->quantity), 2);
			$listing->amount_cents = $totalBucks * 100;
			$listing->title = 'Product: ' . $item->get_name();
			$listing->url = $this->maybeAdjustUrl($product->get_permalink());

			$total += ($listing->amount_cents * $listing->quantity);
			$invoice->listings[] = $listing;
		}

		$invoice->amount_cents = $total;

		return apply_filters('guntab_alter_invoice', $invoice, $order);
	}

	private function maybeAdjustUrl(string $url) : ?string {
		$websiteIsPublic = get_option('guntab_website_is_public', 'yes') === 'yes';
		if (empty($url) || !$this->testmode || $websiteIsPublic || strpos($url, "://") === false) {
			return $url;
		}

		return 'https://example.com';
	}

	public function add_quick_checkout() {
		$guntabSettings = get_option('woocommerce_guntab_settings');
		if (isset($guntabSettings['show_quick_checkout_buttons']) && $guntabSettings['show_quick_checkout_buttons']=='yes') {
			global $product;
			$productId = $product->get_id();

			if ($product->is_type('variable')) {
				echo '
<script>
jQuery(document).ready(function($){
	const $document  = $( document );
	let $submit_name = false;
	$document.on("click", ".guntab_quick_checkout_button", function() {
		selected    = $("form.cart"),
		productform = selected,
		submit_btn  = productform.find( "[type=\'submit\']");
		productform.find( "input[name=guntab_quick_checkout]" ).remove();
		productform.append("<input type=\'hidden\' value=\'true\' name=\'guntab_quick_checkout\' />");

		if( submit_btn.is( \':disabled\' ) ) {
			$( \'html, body\' ).animate( { scrollTop: submit_btn.offset().top - 200 }, 900 );
		} else {
			submit_btn.attr( \'name\', \'wcqb-\' + $submit_name );
			submit_btn.click();
		}
	});
	$(".redirect_to_checkout").click(function(){
		$("button.single_add_to_cart_button ").click();
	});
});
</script>
<div class="button alt guntab_quick_checkout_button" style="cursor:pointer;">Pay with GunTab</div>';
			} elseif ($product->is_type('simple')) {
				echo '
<script>
jQuery(document).ready(function($){
	const $document = $(document);
	$document.on("click", ".redirect_to_checkout", function() {
		window.location.href = "' . wc_get_checkout_url() . '?add-to-cart=' . $productId . '&quantity=" + $(".input-text.qty").val();
	});
});
</script>
<div class="button alt redirect_to_checkout" style="cursor:pointer;">Pay with GunTab</div>';
			}
		}
	}

	public function quick_checkout_redirect() {
		if (isset($_REQUEST['guntab_quick_checkout']) && $_REQUEST['guntab_quick_checkout']) {
			echo "<script>window.location='".wc_get_checkout_url()."'</script>";
			exit;
		}
	}

}
