<?php

namespace GunTab;

use Dragon\Input;
use Dragon\Log;

class GunTabWebhook {
	const STATUS_PENDING_PAYMENT_PROCESSING	= "Pending Payment Processing - Package the order for shipment (but do not send).";
	const STATUS_PENDING_SHIPMENT			= "Pending Shipment - Send the shipment and \"mark as shipped\" in GunTab.";
	const STATUS_PENDING_SHIPMENT_DELIVERY	= "Pending Shipment Delivery - No action necessary, this status is informational. GunTab will notify the buyer of delivery.";
	const STATUS_PENDING_BUYER_INSPECTION_PERIOD	= "Pending Buyer Inspection Period - Expect your disbursement to be sent in 3 days or less.";
	const STATUS_PENDING_DISBURSEMENT_PROCESSING	= "Pending Disbursement Processing - No action necessary, this status is informational. Expect your disbursement to be visible in your online banking in 1-2 business days.";
	const STATUS_PENDING_DISPUTE_REVIEW				= "Pending Dispute Review - The buyer disputed the transaction and GunTab is reviewing.";
	const STATUS_PENDING_REFUND_PROCESSING			= "Pending Refund Processing - The the shipment was returned to sender after being lost or successfully disputed. This is similar to \"Canceled\" status, except the merchandise was probably shipped.";
	
	const STATUS_CANCELED = "Canceled - The buyer decided not to purchase. Most likely the item(s) were not shipped.";
	const STATUS_SUCCEEDED = "Succeeded - Disbursement complete";
	
	private static ?\stdClass $json = null;
	private static ?\WC_Order $order = null;
	
	public static function handleGuntabPaymentWebhook() {
		$input = new Input();
		$json = $input->json();
		if (empty($json) || empty($json->status) || empty($json->transaction_id)) {
			return ["success" => false];
		}
		
		$webhookKey = $input->get('token');
		if (!static::isVerifiedRequest($webhookKey)) {
			Log::info('Blocked webhook request with invalid token. Token given was: ' . $webhookKey);
			return ["success" => false];
		}
		
		if (empty($json->seller_order_id)) {
			return ["success" => false];
		}
		
		static::$json = $json;
		if (method_exists(static::class, 'handle_status_' . $json->status)) {
			$order = wc_get_order((int)$json->seller_order_id);
			if ($order instanceof \WC_Order) {
				static::$order = $order;
				static::{'handle_status_' . $json->status}();
			}
		}
		
		do_action('guntab_on_webhook', $order, $json);
		return ["success" => true];
	}
	
	private static function isVerifiedRequest(?string $key) {
		$webhookKey = get_option('guntab_webhook_key', null);
		return $webhookKey !== null && $webhookKey === $key;
	}
	
	private static function handle_status_pending_payment_processing() {
		static::logStatusChange(static::STATUS_PENDING_PAYMENT_PROCESSING);
		static::$order->payment_complete();
		static::$order->reduce_order_stock();
		
		$newStatus = get_option('guntab_woocommerce_status_payment_in_escrow', 'wc-processing');
		static::$order->update_status($newStatus, __('Payment placed in escrow at GunTab', 'guntab'));
	}
	
	private static function handle_status_pending_shipment() {
		static::logStatusChange(static::STATUS_PENDING_SHIPMENT);
	}
	
	private static function handle_status_pending_shipment_delivery() {
		static::logStatusChange(static::STATUS_PENDING_SHIPMENT_DELIVERY);
	}
	
	private static function handle_status_pending_buyer_inspection_period() {
		static::logStatusChange(static::STATUS_PENDING_BUYER_INSPECTION_PERIOD);
	}
	
	private static function handle_status_pending_disbursement_processing() {
		static::logStatusChange(static::STATUS_PENDING_DISBURSEMENT_PROCESSING);
		$newStatus = static::getFilteredStatus('wc-completed');
		static::$order->update_status($newStatus, __('GunTab is processing your disbursement.', 'guntab'));
	}
	
	private static function handle_status_canceled() {
		static::logStatusChange(static::STATUS_CANCELED);
		$newStatus = static::getFilteredStatus('wc-cancelled');
		static::$order->update_status($newStatus, __('Order canceled on GunTab', 'guntab'));
	}
	
	private static function handle_status_pending_dispute_review() {
		static::logStatusChange(static::STATUS_PENDING_DISPUTE_REVIEW);
		$newStatus = static::getFilteredStatus('wc-on-hold');
		static::$order->update_status($newStatus, __('GunTab dispute in progress...', 'guntab'));
	}
	
	private static function handle_status_pending_refund_processing() {
		static::logStatusChange(static::STATUS_PENDING_REFUND_PROCESSING);
		$newStatus = static::getFilteredStatus('wc-refunded');
		static::$order->update_status($newStatus, __('Refunded through GunTab. If you recieved the item, please adjust your inventory as appropriate.', 'guntab'));
	}
	
	private static function handle_status_succeeded() {
		static::logStatusChange(static::STATUS_SUCCEEDED);
	}
	
	private static function logStatusChange(string $message) {
		Log::info("[ORDER STATUS CHANGE] Order ID #" . static::$order->get_id() . " - $message");
		static::$order->add_order_note($message);
	}
	
	private static function getFilteredStatus(string $suggestedStatus) {
		return apply_filters('guntab_get_new_order_status', $suggestedStatus, static::$json);
	}
}
