<?php

namespace Dragon\Abstracts;

use Dragon\Config;
use Dragon\Input;
use Dragon\Captcha;

class FrontEndFormInvalidNonceException extends \Exception{}
class FrontEndFormMissingRequiredFieldsException extends \Exception{}
class FrontEndFormBadCaptchaException extends \Exception{}

abstract class FrontEndFormAbstract extends PageAbstract
{
	protected $pageData = [];
	protected $button = 'dragon_submit';
	protected $requiredFields = [];
	protected $badNonceMessage = 'An invalid nonce was supplied for the form. Try refreshing the page and reentering your information.';
	protected $missingRequiredMessage = 'One or more required fields are missing. Please enter the missing information and try again.';
	protected $badCaptchaMessage = 'The Captcha field was either incorrect or expired. Please try again.';
	protected $genericError = 'An error occurred.';
	protected $missingField = null;
	protected $hasCaptcha = false;
	protected $input = null;
	
	public function __construct(array $attributes)
	{
		$this->input = new Input();
	}
	
	protected function handleSubmission()
	{
		if ($this->input->hasPost($this->button)) {
			try {
				
				$this->throwIfBadNonce(Config::$namespace . "_nonce");
				$this->throwIfBadCaptcha();
				$this->throwIfMissingRequiredFields();
				return true;
				
			} catch (FrontEndFormInvalidNonceException $e) {
				
				$this->handleBadNonce();
				return false;
				
			} catch (FrontEndFormMissingRequiredFieldsException $e) {
				
				$this->handleMissingRequired();
				return false;
				
			} catch (FrontEndFormBadCaptchaException $e) {
				
				$this->handleBadCaptcha();
				return false;
				
			}
		}
	}
	
	private function throwIfMissingRequiredFields()
	{
		if ($this->areAllRequiredFieldsPresent() === false) {
			throw new FrontEndFormMissingRequiredFieldsException();
		}
	}
	
	private function areAllRequiredFieldsPresent()
	{
		foreach ($this->requiredFields as $field) {
			
			$postNotSet = empty($this->input->post($field));
			$fileNotSet = array_key_exists($field, $_FILES) === false || $_FILES[$field]['tmp_name'] === '';
			
			if ($postNotSet && $fileNotSet) {
				
				$this->missingField = $field;
				return false;
				
			}
			
		}
		
		return true;
	}
	
	private function throwIfBadNonce($pageNamespace)
	{
		if ($this->isValidNonce($pageNamespace) === false) {
			throw new FrontEndFormInvalidNonceException();
		}
	}
	
	private function isValidNonce($pageNamespace)
	{
		return empty($this->input->post($pageNamespace)) ? false : wp_verify_nonce($this->input->post($pageNamespace), $pageNamespace);
	}
	
	private function throwIfBadCaptcha() {
		if ($this->hasCaptcha === false) {
			return;
		}
		
		$captcha = new Captcha();
		if ($captcha->isValid() === false) {
			throw new FrontEndFormBadCaptchaException();
		}
	}
	
	protected function handleBadNonce()
	{
		$this->pageData['notice'] = $this->badNonceMessage;
	}
	
	protected function handleMissingRequired()
	{
		$this->pageData['notice'] = $this->missingRequiredMessage;
	}
	
	protected function handleBadCaptcha()
	{
		$this->pageData['notice'] = $this->badCaptchaMessage;
	}
}
