<?php

namespace Dragon\Abstracts;

use Dragon\Config;
use Dragon\View;
use Dragon\Input;
use Dragon\User;

class OptionsInvalidNonceException extends \Exception{}
class OptionsMissingRequiredFieldsException extends \Exception{}

abstract class OptionsAbstract extends PageAbstract
{
	protected $saveButton = null;
	protected $adminForm = true;
	protected $requiredFields = [];
	protected $optionalFields = [];
	protected $encryptedFields = [];
	protected $successMessage = 'All settings saved successfully.';
	protected $badNonceMessage = 'An invalid nonce was supplied for the form. Try refreshing the page and reentering your information.';
	protected $missingRequiredMessage = 'One or more required fields are missing. Please enter the missing information and try again.';
	protected $genericError = 'An error occurred.';
	protected $pageData = [];
	protected $input = null;
	protected $settingsAreGlobal = true;
	protected $missingFields = [];
	
	protected function saveSettings(&$pageData)
	{
		$this->input = new Input();
		
		if (is_null($this->saveButton)) {
			$this->saveButton = Config::$namespace . '_save';
		}
		
		if (!empty($this->input->post($this->saveButton))) {
			try {
				
				$this->handleSaveSettings();
				if (empty($pageData['notice'])) {
					$pageData['notice'] = $this->getSaveSuccessNotice();
				}
				
			} catch (OptionsInvalidNonceException $e) {
				$pageData['notice'] = $this->getBadNonceNotice();
			} catch (OptionsMissingRequiredFieldsException $e) {
				$pageData['notice'] = $this->getMissingRequiredFieldsNotice();
			} catch (\Exception $e) {
				$pageData['notice'] = View::getNotice('notice-error', $e->getMessage());
			}
		}
	}
	
	protected function updateOptionIfSet($option)
	{
		if ($this->input->hasPost($option)) {
			$this->updateOption($option);
		}
	}
	
	protected function updateOption($option)
	{
		$value = $this->input->post($option);
		if (in_array($option, $this->encryptedFields)) {
			$value = dragonEncrypt($value);
		}
		
		$saveValue = sanitize_textarea_field($value);
		if ($this->settingsAreGlobal) {
			update_option($option, $saveValue);
		} else {
			User::setMeta($option, $saveValue);
		}
	}
	
	protected function save()
	{
		foreach($this->requiredFields as $field) {
			$this->updateOption($field);
		}
		
		foreach($this->optionalFields as $field) {
			$this->updateOptionIfSet($field);
		}
	}
	
	protected function getMissingFields() {
		return $this->missingFields;
	}
	
	private function handleSaveSettings()
	{
		$this->throwIfMissingRequiredFields();
		$this->throwIfBadNonce(Config::$namespace . "_nonce");
		
		$this->save();
	}
	
	private function throwIfMissingRequiredFields()
	{
		if ($this->areAllRequiredFieldsPresent() === false) {
			throw new OptionsMissingRequiredFieldsException();
		}
	}
	
	private function areAllRequiredFieldsPresent()
	{
		$this->missingFields = [];
		foreach ($this->requiredFields as $field) {
			if (($this->input->hasPost($field) === false || $this->input->post($field) === '') &&
				empty($_FILES[$field]['name'])
			) {
				$this->missingFields[] = $field;
			}
		}
		
		return empty($this->missingFields);
	}
	
	private function throwIfBadNonce($pageNamespace)
	{
		if ($this->adminForm === true && $this->isValidNonce($pageNamespace) === false) {
			throw new OptionsInvalidNonceException();
		}
	}
	
	private function isValidNonce($pageNamespace)
	{
		return wp_verify_nonce($this->input->post($pageNamespace), $pageNamespace);
	}
	
	private function getSaveSuccessNotice()
	{
		return View::getNotice('notice-success', __($this->successMessage, Config::$namespace));
	}
	
	private function getBadNonceNotice()
	{
		return View::getNotice('notice-error', __($this->badNonceMessage, Config::$namespace));
	}
	
	private function getMissingRequiredFieldsNotice()
	{
		$missingList = implode(', ', $this->getMissingFields());
		return View::getNotice(
			'notice-error',
			__(
				$this->missingRequiredMessage . '<br />Missing: ' . $missingList,
				Config::$namespace
			)
		);
	}
}
