<?php

namespace Dragon;

use Illuminate\Database\Capsule\Manager as Capsule;
use Dragon\Models\Migration;
use Dragon\Abstracts\MigrationAbstract;

class DB
{
	const MIGRATION_TABLE_NAME = 'dragon_migrations';
	
	private $multisiteNonPrefixedTables = [
		'blogmeta', 'blogs', 'blog_versions',
		'registration_log', 'signups', 'site',
		'sitemeta', 'usermeta', 'users',
	];
	
	private static $instance = null;
	private $migrationDir = null;
	
	public static function make($connections = [])
	{
		if (empty(static::$instance)) {
			static::$instance = new static($connections);
		}
		
		return static::$instance;
	}
	
	public function migrate()
	{
		$this->ensureMigrationsTableExists();
		
		$types = [
			MigrationAbstract::TYPE_CREATE_TABLE,
			MigrationAbstract::TYPE_ADD_COLUMN,
			MigrationAbstract::TYPE_CHANGE_COLUMN,
		];
		
		foreach ($types as $type) {
			
			$this->iterateMigrations(function (MigrationAbstract $migrationClass, $migrationName) use ($type) {
				if ($migrationClass->type === $type &&
					Migration::where('migration', $migrationName)->exists() === false) {
					
					$migrationClass->up();
					Migration::create([
						'migration'	=> $migrationName,
					]);
					
				}
			});
			
		}
	}
	
	public function rollback()
	{
		$this->iterateMigrations(function (MigrationAbstract $migrationClass) {
			
			if ($this->hasTable($migrationClass->tableName)) {
				$migrationClass->down();
			}
			
		});
			
		$this->migrationsTable()->down();
	}
	
	public function hasTable($tableName)
	{
		return Capsule::schema()->hasTable($tableName);
	}
	
	private function __construct($connections)
	{
		global $wpdb;
		$capsule = new Capsule();
		
		$prefix = is_multisite() ? str_replace($wpdb->blogid . '_', '', $wpdb->prefix) : $wpdb->prefix;
		
		$connections = array_merge([
			'default'	=> [
				"driver"	=> "mysql",
				"host"		=> DB_HOST,
				"database"	=> DB_NAME,
				"username"	=> DB_USER,
				"password"	=> DB_PASSWORD,
				"prefix"    => $prefix,
			],
		], $connections);
		
		foreach ($connections as $connectionName => $connectionData) {
			$capsule->addConnection($connectionData, $connectionName);
		}
		
		$capsule->setAsGlobal();
		$capsule->bootEloquent();
		
		$this->setMigrationsDirectory();
	}
	
	private function ensureMigrationsTableExists()
	{
		if ($this->hasTable(static::MIGRATION_TABLE_NAME) === false) {
			$this->migrationsTable()->up();
		}
	}
	
	private function migrationsTable()
	{
		require_once(realpath(__DIR__ . '/../migrations/create_migrations_table.php'));
		return new \CreateMigrationsTable();
	}
	
	private function setMigrationsDirectory()
	{
		$pluginFile = Config::$pluginLoaderFile;
		$this->migrationDir = dirname($pluginFile) . '/migrations/';
	}
	
	private function iterateMigrations(Callable $callback)
	{
		$dir = dir($this->migrationDir);
		while (false !== ($entry = $dir->read())) {
			
			if (in_array($entry, ['.', '..']) || strpos($entry, '.php') === false) {
				continue;
			}
			
			$filename = $this->migrationDir . $entry;
			require_once($filename);
			$migrationName = str_replace('.php', '', $entry);
			$classname = str_replace('_', '', $migrationName);
			
			$callback(new $classname(), $migrationName);
			
		}
		
		$dir->close();
	}
}
